﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 10 $
Author(s): Myrroddin of Llane
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, ChatThrottleLib, AceSerializer-3.0
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 10 $"):match("%d+"))

if not LibStub then error(MAJOR_VERSION .. " requires LibStub.") end
local lib, oldversion = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local _G = getfenv(0)

local libSerial = LibStub("AceSerializer-3.0", true)
local CBH = LibStub("CallbackHandler-1.0")
local CTL = LibStub("ChatThrottleLib")

lib.eventFrame = lib.eventFrame or CreateFrame("Frame")
lib.eventFrame:UnregisterAllEvents()

lib.guildBank = lib.guildBank or {}
lib.callbacks = lib.callbacks or CBH:New(lib)

local ranksCanViewTabs = {}

local function commSend(contents, distribution, target)
	CTL:SendAddonMessage("BULK", "gBankComm", contents, distribution, target)
end

lib:Start()

function lib:Start()
	lib.eventFrame:RegisterEvent("PLAYER_LOGIN")
end

function lib.eventFrame:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
    if prefix ~= "gBankComm" then return end
    if sender == "player" then return end
		for cmd, bankData in msg:Deserialize(string.gmatch(("%a+")("%a+")) do
			if cmd == "REQUEST" then
				local commString:Serialize("UPDATE", lib.guildBank)
				commSend("gBankComm", commString, "WHISPER", sender)
			elseif cmd == "UPDATE" then
				lib.guildBank = bankData
				lib.callbacks:Fire("GuildBankComm_ContentsChanged", sender, bankData)
			end
		end
	end
end

function lib.eventFrame:PLAYER_LOGIN()	
	lib.eventFrame:RegisterEvent("CHAT_MSG_ADDON")
	lib.eventFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
	lib.eventFrame:RegisterEvent("ACCEPT_GUILD")
	lib.eventFrame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED", "ScanPage")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_OPENED")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_CLOSED")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
	
	lib.eventFrame:PLAYER_GUILD_UPDATE()
end

function lib:RequestDataAboutGuild()
	local _, numOnline = GetNumGuildMembers()
	for k = 1, numOnline do
		local _, _, rankIndex = GetGuildInfo(k)		
		local numTabs = GetNumGuildBankTabs()
		_G.GuildControlSetRank(rankIndex..k)
		for i = 1, numTabs do
			local canView = GetGuildBankTabPermissions(numTabs..i)
			if canView then
				ranksCanViewTabs.name = UnitName(k, i)
				ranksCanViewTabs[name].rank = rankIndex..k..i
			end
		end
	end
end

function lib.eventFrame:ACCEPT_GUILD()
	local guildName, _, rankIndex = GetGuildInfo("player")
	local numTabs = GetNumGuildBankTabs()
	_G.GuildControlSetRank(rankIndex)
	for i = 1, numTabs do
		local canView = GetGuildBankTabPermissions(numTabs..i)
		if canView then
			-- only request tabs the player can actually see
			commSend("gBankComm", "REQUEST"..tostring(canView..i), "GUILD")
		end
	end
	lib.guildBank.guildName = guildName
	lib.guildBank[player].rank = rankIndex
end

function lib.eventFrame:PLAYER_GUILD_UPDATE()
	-- check if guild members had their ranks changed
	lib:RequestDataAboutGuild()
	
	-- check if the player's rank has changed
	local oldRank = lib.guildBank[player].rank
	local newRank = _, _, GetGuildInfo("player")
	-- lower numbers are higher rank, IE, 0 is the Guild Master
	if newRank <> oldRank then
		-- force a rescan of the player's bank access
		lib.eventFrame:Accept_GUILD()
	end
	--[===[@debug@
	local guildName, _, rankIndex = GetGuildInfo("player")
	print("The player's guild name is "..guildName.." and player's guild rank number is "..rankIndex)
	--@end-debug@]===]
	
	if not IsInGuild() then -- player left guild			
		local pName = UnitName("player")
		wipe(lib.guildBank)
		local bankData = lib.guildBank
		lib.callbacks:Fire("GuildBankComm_PlayerGuildless", pName, bankData)
	end
end

function lib:GUILDBANKFRAME_OPENED()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
	lib.guildBank[guildName].Money = GetGuildBankMoney()
	
	local name, icon, view	
	for page = 1, lib.guildBank[guildName].numTabs do
		name, icon, view = GetGuildBankTabInfo(page)
		--because I like booleans better just like Starinnia
		if view == 1 then
			view = true
		else
			view = false
		end
		lib.guildBank[guildName].tab.texture[page] = icon
		lib.guildBank[guildName].tab.name[page] = name
		lib.guildBank[guildName].tab.view[page] = view
	end
end

function lib:GUILDBANK_UPDATE_MONEY()
	lib.guildBank[guildName].guildFunds = GetGuildBankMoney()
	--[===[@debug@
	print(lib.guildBank[guildName].." has "..lib.guildBank[guildName].guildFunds.." in copper")
	--@end-debug@]===]
end

function lib:GUILDBANKFRAME_CLOSED()
	local commString:Serialize("UPDATE", lib.guildBank)
	for i = 1, #ranksCanViewTabs[ranksCanViewTabs] in ipairs(ranksCanViewTabs) do
		local guildie = ranksCanViewTabs.name..i		
		commSend(string.format("gBankComm", commString), "WHISPER", guildie)
	end
	local pName = UnitName("player")
	local bankData = lib.guildBank
	lib.CallBacks:Fire("GuildBankComm_ContentsChanged", pName, bankData)
end

function lib:GUILDBANK_UPDATE_TABS()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
end

local num, itemString
function lib:ScanPage()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[guildName].links[page][slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[guildName].info[page][slot] = num
			else
				--if there is no link, slot is empty... fill slot w/ empty string
				--needed to keep consistent tables with keys from 1 to 98
				lib.guildBank[guildName].links[page][slot] = "empty"
				lib.guildBank[guildName].info[page][slot] = "empty"
			end
			num, itemString = nil, nil
		end
		-- time stamp
		lib.guildBank[guildName].lastScan = GetTime()
	end
end